@@ -4,10 +4,13 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <unistd.h>
+#include <tlsComm.h>
 
 int debug_default = 2;
 int SkipCertificateCheck = 0;
 const char *certificate_filename = NULL;
+const char *tls = "NORMAL";
+
 int exists(const char *filename __attribute__ ((unused)))
 {
 	return (0);
