@@ -17,7 +17,7 @@
         * Now supports >256 colors
 	11/09/1998 (Martijn Pieterse, pieterse@xs4all.nl)
 		* Removed a bug from parse_rcfile. You could
-		  not use "start" in a command if a label was 
+		  not use "start" in a command if a label was
 		  also start.
 		* Changed the needed geometry string.
 		  We don't use window size, and don't support
@@ -189,7 +189,7 @@ static void GetXPM(XpmIcon * wmgen_local, const char *pixmap_bytes[])
 
 	/* For the colormap */
 	XGetWindowAttributes(display, Root, &attributes);
-	/* despite the comment, I still don't understand... 
+	/* despite the comment, I still don't understand...
 	   attributes is subsequently unused in this function -ns 11/2002 */
 
 	wmgen_local->attributes.valuemask |=
@@ -469,12 +469,17 @@ void openXwindow(int argc, const char *argv[],
 	int dummy = 0;
 	int i;
 
+	if (!wname) {
+		fprintf(stderr, "Unable to allocate memory for window name!\n");
+		abort();
+	}
+
 	for (i = 1; argv[i]; i++) {
-		if (!strcmp(argv[i], "-display")) {
+		if (!strcmp(argv[i], "-display") && i < argc - 1) {
 			display_name = argv[i + 1];
 			i++;
 		}
-		if (!strcmp(argv[i], "-geometry")) {
+		if (!strcmp(argv[i], "-geometry") && i < argc - 1) {
 			geometry = argv[i + 1];
 			i++;
 		}
@@ -543,6 +548,7 @@ void openXwindow(int argc, const char *argv[],
 	}
 
 	XSetWMName(display, win, &name);
+	XFree(name.value);
 
 	/* Create GC for drawing */
 
@@ -592,7 +598,7 @@ void openXwindow(int argc, const char *argv[],
 		   if( specified & YNegative ) {
 		   y = DisplayHeight(display, DefaultScreen(display)) - y - pixmask_height;
 		   }
-		   if( specified & XValue || specified & YValue ) { 
+		   if( specified & XValue || specified & YValue ) {
 		   XMoveWindow(display, win, x, y);
 		   } */
 
@@ -609,4 +615,7 @@ void openXwindow(int argc, const char *argv[],
 		   exit(1);
 		   } */
 	}
+
+	if (wname)
+		free(wname);
 }
